@@ -1,4 +1,4 @@
-'''Features and tools for resolving sequences of images.
+"""Features and tools for resolving sequences of images.
 
 Classes
 -------
@@ -9,15 +9,14 @@ Functions
 ---------
 Sequential
     Converts a feature to be resolved as a sequence.
-'''
+"""
 
 from deeptrack.features import Feature
 from deeptrack.properties import SequentialProperty
 
 
-
 class Sequence(Feature):
-    ''' Resolves a feature as a sequence.
+    """Resolves a feature as a sequence.
 
     The input feature is resolved `sequence_length` times, with the kwarg
     arguments `sequene_length` and `sequence_step` passed to all properties
@@ -34,7 +33,7 @@ class Sequence(Feature):
     ----------
     feature : Feature
         The feature to resolve as a sequence.
-    '''
+    """
 
     __distributed__ = False
 
@@ -46,11 +45,12 @@ class Sequence(Feature):
         self.update()
 
     def get(self, input_list, sequence_length=None, **kwargs):
-        return (input_list or []) + [self.feature.resolve(
-                                sequence_length=sequence_length, 
-                                sequence_step=sequence_step,
-                                **kwargs)
-                for sequence_step in range(sequence_length)]
+        return (input_list or []) + [
+            self.feature.resolve(
+                sequence_length=sequence_length, sequence_step=sequence_step, **kwargs
+            )
+            for sequence_step in range(sequence_length)
+        ]
 
     def update(self, **kwargs):
         super().update(**kwargs)
@@ -64,9 +64,8 @@ class Sequence(Feature):
         return self
 
 
-
 def Sequential(feature: Feature, **kwargs):
-    '''Converts a feature to be resolved as a sequence.
+    """Converts a feature to be resolved as a sequence.
 
     Should be called on individual features, not combinations of features. All keyword
     arguments will be trated as sequential properties and will be passed to the parent feature.
@@ -81,8 +80,7 @@ def Sequential(feature: Feature, **kwargs):
     kwargs
         Keyword arguments to pass on as sequential properties of `feature`.
 
-    '''
-
+    """
 
     for property_name, sampling_rule in kwargs.items():
 
@@ -90,9 +88,10 @@ def Sequential(feature: Feature, **kwargs):
             initializer = feature.properties[property_name].sampling_rule
         else:
             initializer = sampling_rule
-            
-        feature.properties[property_name] = SequentialProperty(sampling_rule, initializer=initializer)
+
+        feature.properties[property_name] = SequentialProperty(
+            sampling_rule, initializer=initializer
+        )
         feature.properties[property_name].parent = feature.properties
 
     return feature
-
